# Microeconometrics
# CLASS 3
# Count data / ordered responses
# Wiktor Budzinski, Marek Giergiczny

# ctrl+l
rm(list=ls())
setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 3")        # sets the access path (sometimes change the characters '\' to '/' is needed)

# Installing required packages
packages = c("psych",  "MASS", "oglmx", "pscl")
find.package(packages)

# if the library is not installed on the given computer, delete the first '#'
# install.packages("psych")
# install.packages("MASS")
# install.packages("oglmx")
# install.packages("pscl")
rm(packages)

# Loading libraries
library(psych)
library(statmod)
# library(dummies)
library(MASS)
library(mfx)
library(oglmx)
library(pscl)
library(msm)
library(nlWaldTest)
library(glmx)


################################################# Exercise 1 ########################################
# Ordered models

rm(list=ls())
baltic  <-  readRDS("me.baltic.rds")
baltic <- baltic[baltic$ruo == 0,]
baltic$age <- baltic$age/10
hist(baltic$envw)

Eq1 <- factor(envw) ~ dk + ee + fi + lit + lv + pl + ruc + se + bocc + age + male + factor(edu) + hhsize + hinc + ldist
model1 <- polr(Eq1, data=baltic, Hess=TRUE, method = c("logistic"))
summary(model1)

# Alternative method (with threshold parameters)
Eq1b <-  envw ~ dk + ee + fi + lit + lv + pl + ruc  + se + bocc + age + male + factor(edu) + hhsize + hinc + ldist
model1b <- ologit.reg(Eq1b, data=baltic)
summary(model1b)
# Interpretation
# People whose profession is associated with the Baltic Sea are more worried (on average) about the state of its environment

# Marginal effects
options(scipen=999) # disable scientific notation
margins.oglmx(model1b, AME=TRUE) # average marginal effects

margins.oglmx(model1b) # at mean
margins.oglmx(model1b, ascontinuous = TRUE)  # binary variable are treated as if continous

# Similarly, you can estimate an ordered probit

model2 <- oprobit.reg(Eq1b, data=baltic)
summary(model2) 

################################################# Exercise 2 ########################################
# Heteroskedasticity

model3 <-oglmx(Eq1b, ~dk + ee + fi + lit + lv + pl + ruc  + se, data=baltic,  constantSD=FALSE, constantMEAN=FALSE)
summary(model3) 

################################################# Exercise 3 #########################################
# Binary data

rm(list=ls())

oil  =  readRDS("me.oil1.rds")
View(oil)
hist(oil$bid)
Eq1 <- ans ~ bid + linc + notax +ccoast
model1 <- glm(Eq1, data = oil, family=binomial(link="logit"))
summary(model1)

model1b <- ologit.reg(Eq1, data=oil)
summary(model1b)

# Marginal effects for logit
me1 <- logitmfx(Eq1, data = oil, atmean = FALSE)
me1

# WTP for the program

nlConfint(model1, "-b[1]/b[2]")
mean(oil$linc); mean(oil$notax); mean(oil$ccoast)

nlConfint(model1, "-(b[1]+b[3]*10.43795)/b[2]") # average income
nlConfint(model1, "-(b[1]+b[3]*10.43795 + b[5])/b[2]") #at the coast

# Different types of functional forms
model2 <- glm(Eq1, data = oil, family=binomial(link="probit"))
summary(model2)

extractAIC(model1); extractAIC(model2);

################################################# Exercise 4 ########################################

rm(list=ls())
baltic  <-  readRDS("me.baltic.rds")
baltic <- baltic[baltic$ruo == 0,]
baltic$tc <- (baltic$tc_km + baltic$tc_time/3)/10
baltic$age <- baltic$age/10

Eq1 <- trips ~ dk + ee + fi + lit + lv + pl + ruc + se + age + bocc+ male + factor(edu) + hhsize+  tc
model1 <- glm(Eq1, data=baltic, family = poisson)
summary(model1)
# Qualitative interpretation - the higher the travel cost, the less trips to the Baltic Sea

# Quantitative interpretation: marginal effects

me1<-poissonmfx(Eq1, data=baltic ,atmean = FALSE)
me1$mfxest

# Consumer surplus for one visit (in Euro)
length(coef(model1))
-1 / (coef(model1)[17])
-10 / (coef(model1)[17])

betas <- coef(model1)
V<-vcov(model1)
# Obataining standard errors
deltamethod(~ -10/x17, betas, V) 

# alternatively, calculate with confidence intervals
nlConfint(model1, "-10/b[17]")

# Consumer surplus for each country
baltic$lambda <- model1$fitted.values
-10*mean(model1$fitted.values[baltic$de == 1]/(coef(model1)[17])) # Germany has lower CS because of tc
-10*mean(model1$fitted.values[baltic$dk == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$ee == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$fi == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$lit == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$lv == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$pl == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$ruc == 1]/(coef(model1)[17]))
-10*mean(model1$fitted.values[baltic$se == 1]/(coef(model1)[17]))

################################################# Exercise 5 ########################################
# Diagnostics for Poisson

# Linktest
baltic$xb <- log(baltic$lambda)
summary(glm(trips ~ xb + I(xb^2), data=baltic, family = poisson))

# Residuals
baltic$eQ <- qresid(model1)
hist(baltic$eQ)

scatter.smooth(baltic$eQ[!is.nan(baltic$eQ) & is.finite(baltic$eQ)] ~ baltic$tc[!is.nan(baltic$eQ) & is.finite(baltic$eQ)], col="grey",
               las=1, ylab="Quantile residuals", xlab="Travel cost")


# Predicted (expected) number of visit
baltic$z <- ((baltic$trips - baltic$lambda)^2 - baltic$trips) / baltic$lambda
model1b <- lm(z ~ lambda - 1, data=baltic)
summary(model1b)
hist(baltic$trips)
# the parameter is statistically significant - H0 (equi-dispersion) should be rejected

# It turns out, that variance  is significantly higher than the expected value
hist(baltic$trips)
describe(baltic$trips)

################################################# Exercise 6 ########################################
# Construct a binomial negative regression model


model2 <- glm.nb(Eq1, data=baltic)
summary(baltic$tc)
baltic$tc <- baltic$tc/10
model2 <- glm.nb(Eq1, data=baltic)
summary(model2)
Eq1b <- trips ~ dk + ee + fi + lit + lv + pl + ruc + se + age + bocc+ male + factor(edu) + hhsize
model2b <- glm.nb(Eq1b, data=baltic)
start<-coef(model2b)
start<-append(start, 0.001)
names(start)[17] <-"tc"
names(start)
model2 <- glm.nb(Eq1, data=baltic, start = start)
quantile(baltic$tc, 0.95)
baltic <-baltic[baltic$tc < quantile(baltic$tc, 0.95), ]
model2 <- glm.nb(Eq1, data=baltic, start = start)
summary(model2)

model1c <- glm(Eq1, data=baltic, family = poisson)
extractAIC(model1c); extractAIC(model2)

# Likelihood ratio test
LR <- 2 * (logLik(model2) - logLik(model1c))
pchisq(LR, df = 1, lower.tail = FALSE)

lrtest(model1c,model2)

# 1/theta = 0
nlWaldtest(model2, "1/b[1]=0", coeff = model2$theta, Vcov = model2$SE.theta^2)

# Confidence interval
nlConfint(model2, "1/b[1]", level = 0.95, coeff = model2$theta, Vcov = model2$SE.theta^2)
  hist(baltic$trips)
  mean(baltic$trips == 0)
  
################################################# Exercise 7 ########################################
# Zero inflated models

Eq2 <- trips ~ dk + ee + fi + lit + lv + pl + ruc + se + age + bocc+ male + factor(edu) + hhsize+tc | dk + ee + fi + lit + lv + pl + ruc + se

model3 <- zeroinfl(Eq2, dist="poisson", data=baltic)
summary(model3)
vuong(model1c, model3)

model4 <- zeroinfl(Eq2, dist="negbin", data=baltic)
summary(model4)

model4b <- zeroinfl(Eq2, dist="negbin", data=baltic, 
                    start = list(count = model3$coefficients$count, zero = model3$coefficients$zero/10, theta = 0.25))
summary(model4b)

model4c <- zeroinfl(Eq2, dist="negbin", data=baltic, 
                    start = list(count = model3$coefficients$count/15, zero = model3$coefficients$zero/20, theta = 0.1))
summary(model4c)

logLik(model4); logLik(model4b); logLik(model4c)
vuong(model2, model4b)

# Compare ZIP with ZINB
LR2 <- 2 * (logLik(model4b) - logLik(model3))
pchisq(LR2, df = 1, lower.tail = FALSE)

################################################# Exercise 8 ########################################
# Hurdle models

model5 <- hurdle(Eq2, dist="poisson", data=baltic)
summary(model5)
vuong(model1c, model5)
model5b <- hurdle(Eq2, dist="poisson", zero.dist = "poisson", data=baltic)
summary(model5b)
extractAIC(model5); extractAIC(model5b)

Eq2b <- trips ~ dk + ee + fi + lit + lv + pl + ruc + se + age + bocc+ male + factor(edu) + hhsize+tc | dk + ee + fi + lit + lv + pl + ruc + se+tc
model5c <- hurdle(Eq2b, dist="poisson", data=baltic)
summary(model5c)
model5d <- hurdle(Eq2b, dist="poisson", zero.dist = "poisson", data=baltic)
summary(model5d)
extractAIC(model5c); extractAIC(model5d)

Eq2c <- trips ~ dk + ee + fi + lit + lv + pl + ruc + se + age + bocc+ male + factor(edu) + hhsize+tc
model5e <- hurdle(Eq2c, dist="poisson", zero.dist = "poisson", data=baltic)
summary(model5e)
hurdletest(model5e)

model6 <- hurdle(Eq2, dist="negbin", data=baltic)
summary(model5)
vuong(model2, model6)

extractAIC(model3); extractAIC(model4b); extractAIC(model5); extractAIC(model6)