
library(pscl)
library(tseries)
library(nlWaldTest)

## Simulation Example no 1.
# CVM

rm(list=ls())
set.seed(1234567) # setting seed

WTP <- rpois(1000, 20)
hist(WTP)

bid<- sample.int(5, 1000, replace = TRUE)
summary(WTP)
bid[bid == 5] = 30
bid[bid == 4] = 25
bid[bid == 3] = 20
bid[bid == 2] = 15
bid[bid == 1] = 10
hist(bid)
ans <- WTP > bid

model1 <- glm(ans ~ bid,  family=binomial(link="logit"))
summary(model1)

# WTP for the program
nlConfint(model1, "-b[1]/b[2]")
mean(WTP)

## Simulation Example no 2.

set.seed(1234567) # setting seed
x1 <- runif(1000,-2,2)
y<-rpois(1000, exp(0.5+1*x1))
m <- glm(y~x1,  family = poisson)
lambda <- m$fitted.values

z <- ((y - lambda)^2 - y) / lambda
mx <- lm(z ~ lambda - 1)
summary(mx)

## Simulation Example no 3.

set.seed(1234567) # setting seed

# Using Voung test for comparing Poisson and ZIP

x1 <- runif(1000,-2,2)
y<-rpois(1000, exp(0.5+1*x1))
hist(y)

m1 <- glm(y~x1,  family = poisson)
summary(m1)
m2 <- zeroinfl(y~x1, dist="poisson")
summary(m2)
vuong(m2, m1)

lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1)
sum(lp)
logLik(m1)

lam2 <- predict(m2, type = "count")
p0 <- predict(m2, type = "zero")
p2_tmp<- y*log(lam2)-lam2-lgamma(y+1)
p2 <- (1-p0)*exp(p2_tmp)
p2[y==0] <- p2[y==0] + p0[y==0]
lp2<- log(p2)
sum(lp2)
logLik(m2)

V<- sqrt(1000)*mean(lp2-lp)/sd(lp2-lp)
V

# Actual simulation 

iter <- 1000                        # number of iterations
Vmat <- matrix(, nrow = iter)
N<-1000
start_time = Sys.time()

for (i in 1:iter) {
  
  x1 <- runif(N,-2,2)
  y<-rpois(N, exp(0.5+1*x1))
  m1 <- glm(y~x1,  family = poisson)
  m2 <- zeroinfl(y~x1, dist="poisson")
  
  lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1)
  lam2 <- predict(m2, type = "count")
  p0 <- predict(m2, type = "zero")
  p2_tmp<- y*log(lam2)-lam2-lgamma(y+1)
  p2 <- (1-p0)*exp(p2_tmp)
  p2[y==0] <- p2[y==0] + p0[y==0]
  lp2<- log(p2)
  
  Vmat[i]<- sqrt(N)*mean(lp2-lp)/sd(lp2-lp)
  
}
end_time = Sys.time()
end_time -start_time
hist(Vmat)
jarque.bera.test(Vmat)

## For comparison:

rm(list=ls())
set.seed(1234567) # setting seed

x1 <- runif(1000,-2,2)
x2 <- runif(1000,-2,2)
y<-rpois(1000, exp(0.5+1*x1 + x2))
hist(y)

m1 <- glm(y~x1,  family = poisson)
summary(m1)
m2 <- glm(y~x2,  family = poisson)
summary(m2)
vuong(m2, m1)

lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1)
sum(lp)
logLik(m1)

lp2<-y*log(m2$fitted.values)-m2$fitted.values-lgamma(y+1)
sum(lp2)
logLik(m2)

V<- sqrt(1000)*mean(lp2-lp)/sd(lp2-lp)
V

# Actual simulation 

iter <- 1000                        # number of iterations
Vmat <- matrix(, nrow = iter)
N<-1000
start_time = Sys.time()

for (i in 1:iter) {
  
  x1 <- runif(1000,-2,2)
  x2 <- runif(1000,-2,2)
  y<-rpois(1000, exp(0.5+1*x1 + x2))
  
  m1 <- glm(y~x1,  family = poisson)
  m2 <- glm(y~x2,  family = poisson)
  
  lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1)
  lp2<-y*log(m2$fitted.values)-m2$fitted.values-lgamma(y+1)
  
  Vmat[i]<- sqrt(N)*mean(lp2-lp)/sd(lp2-lp)
}
end_time = Sys.time()
end_time -start_time
hist(Vmat)
jarque.bera.test(Vmat)

